# Configuration Reference

This section is the authoritative reference for JanusGraph configuration
options. It includes all options for storage and indexing backends that
are part of the official JanusGraph distribution.

The table is automatically generated by traversing the keys and
namespaces in JanusGraph’s internal configuration management API. Hence,
the configuration options as listed on this page are synchronized with a
particular JanusGraph release. If a reference to a configuration option
in other parts of this documentation is in conflict with its
representation on this page, assume the version listed here to be
correct.

## Mutability Levels

Each configuration option has a certain mutability level that governs
whether and how it can be modified after the database is opened for the
first time. The following listing describes the mutability levels.

* FIXED

    Once the database has been opened, these configuration options cannot be changed for the entire life of the database

* GLOBAL\_OFFLINE

    These options can only be changed for the entire database cluster at once when all instances are shut down

* GLOBAL

    These options can only be changed globally across the entire database cluster

* MASKABLE

    These options are global but can be overwritten by a local configuration file

* LOCAL
    These options can only be provided through a local configuration file

Refer to [Global Configuration](#configuration-global) for information
on how to change non-local configuration options.

## Umbrella Namespace

Namespaces marked with an asterisk are **umbrella namespaces** which
means that they can accommodate an arbitrary number of sub-namespaces -
each of which uniquely identified by its name. The configuration options
listed under an umbrella namespace apply only to those sub-namespaces.
Umbrella namespaces are used to configure multiple system components
that are of the same type and hence have the same configuration options.

For example, the `log` namespace is an umbrella namespace because
JanusGraph can interface with multiple logging backends, such as the
`user` log, each of which has the same core set of configuration
options. To configure the send batch size of the `user` log to 100
transaction changes, one would have to set the following option in the
configuration
```properties
log.user.send-batch-size = 100
```

## Configuration Namespaces and Options
{!basics/janusgraph-cfg.md!}